<?php

namespace Drupal\views_data_export_phpspreadsheet\Encoder;

use Drupal\Component\Utility\Html;
use Symfony\Component\Serializer\Encoder\DecoderInterface;
use Symfony\Component\Serializer\Encoder\EncoderInterface;
use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
/**
 * Adds Xls encoder support for the Serialization API.
 */
class XlsEncoder implements EncoderInterface, DecoderInterface {

  /**
   * The format that this encoder supports.
   *
   * @var string
   */
  protected static $supportFormat = 'xls';
  /**
   * Store current row number in spreadsheet
   *
   * @var int
   */
  private $spreadsheet_current_row = 1;

  public $format = 'xlsx';
  /**
   * Constructs an XLS encoder.
   *
   */
  public function __construct($format = 'xlsx', $context = FALSE) {
    $this->format = $format;
  }

  /**
   * The PhpSpreadsheet object, if it's initialized.
   *
   * @var \PhpOffice\PhpSpreadsheet\Spreadsheet
   */
  private $spreadsheet;

  /**
   * {@inheritdoc}
   */
  public function supportsEncoding($format, $context = FALSE) {
    return $format == static::$supportFormat;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsDecoding($format, $context = FALSE) {
    return $format == static::$supportFormat;
  }

  /**
   * {@inheritdoc}
   */
  public function decode($data, $format, array $context = []) {
    $reader = \PhpOffice\PhpSpreadsheet\IOFactory::createReader(ucfirst($this->format));
    $spreadsheet = $reader->loadFromString($data);
    $sheetIndicators = $spreadsheet->getActiveSheet();;
    $sheetData = null;
    if (null !== $sheetIndicators) {
      $sheetData = $sheetIndicators->toArray(null, true, true, true);
    }
    return $sheetData;
  }

  /**
   * {@inheritdoc}
   */
  public function encode($data, $format, array $context = []) {
    switch (gettype($data)) {
      case 'array':
        break;
      case 'object':
        $data = (array) $data;
        break;
      default:
        $data = [$data];
        break;
    }
    $header = FALSE;
    $footer = FALSE;
    $fields = null;
    $colors = null;
    $autoSize = null;
    $dataTypes = null;
    $title = '';
    $totalCol = 0;
    if(!empty($context["views_style_plugin"]) && !empty($options = $context["views_style_plugin"]->options)){
      if(!empty($options["xls_settings"]["xls_format"])){
        $this->format = $options["xls_settings"]["xls_format"];
      }
      $view = $context["views_style_plugin"]->view;
      if(!empty($view->preview)){
        return \Drupal\csv_serialization\Encoder\CsvEncoder::encode($data, $format, $context);
      }
      if(empty($view->getOffset())){
        $index = 1;
        foreach ($view->field as $fieldName => $field) {
          if($field->options['exclude']){
            continue;
          }
          $fields[$fieldName] = !empty($field->options['label']) ? $field->options['label'] : (string) $field->definition['title'];
          //set color background to the column
          $col_name = \PhpOffice\PhpSpreadsheet\Cell\Coordinate::stringFromColumnIndex($index);
          if (!empty($options["xls_settings"]["color"][$fieldName])) {
            $colorSet = ltrim($options["xls_settings"]["color"][$fieldName], '#');
            if (!in_array($colorSet, ['000', '000000'])) {
              $col_coordinate = $col_name ;//. '1' . $col_name . $view->total_rows;
              $colors[$col_coordinate] = $colorSet;
            }
          }
          //set column auto size
          $autoSize[$col_name] = TRUE;
          $index++;
        }
        $totalCol = count($fields);
        //add column header
        if(!empty($fields)){
          array_unshift($data, $fields);
        }
        //add print header
        if(!empty($options["xls_settings"]["header"]["value"])){
          $header = $view->getStyle()->tokenizeValue($options["xls_settings"]["header"]["value"], 0);
          $header = \Drupal::token()->replace($header, ['view' => $view]);
          array_unshift($data, [$header]);
        }
        //add print footer
        if(!empty($options["xls_settings"]["footer"]["value"])) {
          $footer = $view->getStyle()->tokenizeValue($options["xls_settings"]["footer"]["value"], 0);
          $footer = \Drupal::token()->replace($footer, ['view' => $view]);
        }
        if (!empty($view->getTitle())) {
          $title = $view->getTitle();
        }
      }
    }
    try {
      //Create a new excel object.
      $spreadsheet = $this->createSpreadsheet();
      $worksheet = $spreadsheet->getActiveSheet();
      //set worksheet name
      if (!empty($title)) {
        $worksheet->setTitle(preg_replace('/[^\da-z ]/i', '', $title));
      }
      //Print header
      if ($header) {
        $worksheet->getHeaderFooter()->setOddHeader('&C&H' . $header);
        $worksheet->mergeCellsByColumnAndRow(1, 1, $totalCol, 1);
      }
      //Print footer with pagination
      if ($footer) {
        $worksheet->getHeaderFooter()->setOddFooter('&L&B' . $footer . '&RPage &P of &N');
      }
      //fill color
      if ($colors) {
        foreach ($colors as $coordiante => $color) {
          $worksheet->getStyle($coordiante)
            ->getFill()
            ->setFillType(\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID)
            ->getStartColor()
            ->setARGB($color);
        }
      }
      //setauto size
      if ($autoSize) {
        foreach ($autoSize as $coordiante => $active) {
          $worksheet->getColumnDimension($coordiante)->setAutoSize($active);
        }
      }
      //set document properties
      if(!empty($options['xls_settings']['metadata']) && empty($view->offset)) {
        $spreadsheet->getProperties()
          ->setCreator($options['xls_settings']['metadata']['creator'])
          ->setLastModifiedBy($options['xls_settings']['metadata']['last_modified_by'])
          ->setTitle($options['xls_settings']['metadata']['title'])
          ->setSubject($options['xls_settings']['metadata']['subject'])
          ->setDescription($options['xls_settings']['metadata']['description'])
          ->setKeywords($options['xls_settings']['metadata']['keywords'])
          ->setCategory($options['xls_settings']['metadata']['category'])
          ->setManager($options['xls_settings']['metadata']['manager'])
          ->setCompany($options['xls_settings']['metadata']['company']);
      }
      //generate data
      foreach ($data as $row => $rowData) {
        $column = 1;
        foreach ($rowData as $fieldName => $value) {
          $worksheet->setCellValueByColumnAndRow($column++, $row+1, $this->formatValue($value));
        }
      }

      $writer = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, ucfirst($this->format));
      ob_start();
      $writer->save('php://output');
      $spreadsheet->disconnectWorksheets();
      return ob_get_clean();
    }
    catch (\Exception $e) {
      throw new InvalidDataTypeException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * Create a new PhpSpreadsheet.
   */
  public function createSpreadsheet() {
    $spreadsheet = new Spreadsheet();
    $spreadsheet->setActiveSheetIndex(0);
    return $spreadsheet;
  }
  /**
   * filter value
   */
  protected function formatValue($value) {
    $value = Html::decodeEntities($value);
    $value = strip_tags($value);
    $value = trim($value);
    return $value;
  }
}
